@@ -1,7 +1,5 @@
 /*
-  $Id: isofile2.cpp,v 1.2 2008/03/24 15:30:56 karl Exp $
-
-  Copyright (C) 2004, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2006, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,18 +29,11 @@
 
 #define LOCAL_FILENAME "copying"
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #include "portable.h"
 
-#include <sys/types.h>
-#include <cdio/cdio.h>
-#include <cdio/iso9660.h>
-
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
-
+#endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -59,6 +50,9 @@
 #include <sys/types.h>
 #endif
 
+#include <cdio/cdio.h>
+#include <cdio/iso9660.h>
+
 #define CEILING(x, y) ((x+(y-1))/y)
 
 #define my_exit(rc)				\
