@@ -1,6 +1,6 @@
 /*
-  Copyright (C) 2005, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
-  
+  Copyright (C) 2005, 2008, 2009, 2017 Rocky Bernstein <rocky@gnu.org>
+
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
@@ -55,39 +55,39 @@
 int
 main(int argc, const char *argv[])
 {
-  CdioList_t *entlist;
+  CdioISO9660FileList_t *p_entlist;
   CdioListNode_t *entnode;
   char const *psz_fname;
   iso9660_t *p_iso;
 
-  if (argc > 1) 
+  if (argc > 1)
     psz_fname = argv[1];
-  else 
+  else
     psz_fname = ISO9660_IMAGE;
 
   p_iso = iso9660_open_fuzzy (psz_fname, 5);
-  
+
   if (NULL == p_iso) {
-    fprintf(stderr, "-- Sorry, could not find an ISO 9660 image from %s\n", 
+    fprintf(stderr, "-- Sorry, could not find an ISO 9660 image from %s\n",
 	    psz_fname);
     return 1;
   }
 
-  entlist = iso9660_ifs_readdir (p_iso, "/");
-    
+  p_entlist = iso9660_ifs_readdir (p_iso, "/");
+
   /* Iterate over the list of nodes that iso9660_ifs_readdir gives  */
 
-  if (entlist) {
-    _CDIO_LIST_FOREACH (entnode, entlist)
+  if (p_entlist) {
+    _CDIO_LIST_FOREACH (entnode, p_entlist)
     {
       char filename[4096];
-      iso9660_stat_t *p_statbuf = 
+      iso9660_stat_t *p_statbuf =
 	(iso9660_stat_t *) _cdio_list_node_data (entnode);
       iso9660_name_translate(p_statbuf->filename, filename);
       printf ("-- /%s\n", filename);
     }
 
-    _cdio_list_free (entlist, true);
+    iso9660_filelist_free(p_entlist);
   }
 
   iso9660_close(p_iso);
