@@ -2,7 +2,7 @@
    Wrappers for specific Multimedia Command (MMC) commands e.g., READ
    DISC, START/STOP UNIT.
    
-   Copyright (C) 2010, 2011 Rocky Bernstein <rocky@gnu.org>
+   Copyright (C) 2010-2012 Rocky Bernstein <rocky@gnu.org>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,10 @@
 # define __CDIO_CONFIG_H__ 1
 #endif
 
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
+
 #include <cdio/cdio.h>
 #include <cdio/mmc_ll_cmds.h>
 #include "cdio_private.h"
@@ -33,23 +37,40 @@
 #endif
 
 /**
-  Get drive capabilities vis SCSI-MMC GET CONFIGURATION
-  @param p_cdio the CD object to be acted upon.
-  @return DRIVER_OP_SUCCESS (0) if we got the status.
-  return codes are the same as driver_return_code_t
+   Get drive capabilities vis SCSI-MMC GET CONFIGURATION
+   @param p_cdio the CD object to be acted upon.
+   
+   @param p_buf pointer to location to store mode sense information
+   @param i_size number of bytes allocated to p_buf
+   @param i_return_type value in range 0..2 giving what kind of configuration
+   to return:
+   
+   : 0 Full Header and Full Descriptors; 
+   : 1 Feature Headers and those with their Current Bit. 
+   : 2 One Feature header and zero or one Feature Descriptors.
+   
+   @param i_starting_feature_number feature number from which to start
+   getting information.
+   
+   @param i_timeout_ms value in milliseconds to use on timeout. Setting
+   to 0 uses the default time-out value stored in
+   mmc_timeout_ms.      
+   
+   @return DRIVER_OP_SUCCESS (0) if we got the status.
+   return codes are the same as driver_return_code_t
 */
 driver_return_code_t 
 mmc_get_configuration(const CdIo_t *p_cdio, void *p_buf, 
-		      unsigned int i_size, 
-		      unsigned int return_type, 
-		      unsigned int i_starting_feature_number,
-		      unsigned int i_timeout_ms)
+                      unsigned int i_size, 
+                      unsigned int i_return_type, 
+                      unsigned int i_starting_feature_number,
+                      unsigned int i_timeout_ms)
 
 {
     MMC_CMD_SETUP(CDIO_MMC_GPCMD_GET_CONFIGURATION);
     CDIO_MMC_SET_READ_LENGTH8(cdb.field, i_size);
     if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
-    cdb.field[1] = return_type & 0x3;
+    cdb.field[1] = i_return_type & 0x3;
     CDIO_MMC_SET_LEN16(cdb.field, 2, i_starting_feature_number);
     return MMC_RUN_CMD(SCSI_MMC_DATA_READ, i_timeout_ms);
 }
@@ -79,8 +100,8 @@ mmc_get_event_status(const CdIo_t *p_cdio, uint8_t out_buf[2])
     
     i_status = MMC_RUN_CMD(SCSI_MMC_DATA_READ, mmc_timeout_ms);
     if (i_status == DRIVER_OP_SUCCESS) {
-	out_buf[0] = buf[4];
-	out_buf[1] = buf[5];
+        out_buf[0] = buf[4];
+        out_buf[1] = buf[5];
     }
     return i_status;
 }
@@ -98,14 +119,14 @@ mmc_get_event_status(const CdIo_t *p_cdio, uint8_t out_buf[2])
    
    @param i_timeout_ms value in milliseconds to use on timeout. Setting
           to 0 uses the default time-out value stored in
-	  mmc_timeout_ms.
+          mmc_timeout_ms.
 
    @return DRIVER_OP_SUCCESS if we ran the command ok.
 
  */
 driver_return_code_t
 mmc_mode_select_10(CdIo_t *p_cdio, /*out*/ void *p_buf, unsigned int i_size, 
-		   int page, unsigned int i_timeout_ms)
+                   int page, unsigned int i_timeout_ms)
 {
     MMC_CMD_SETUP_READ16(CDIO_MMC_GPCMD_MODE_SELECT_10);
     if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
@@ -124,7 +145,7 @@ mmc_mode_select_10(CdIo_t *p_cdio, /*out*/ void *p_buf, unsigned int i_size,
 */
 driver_return_code_t
 mmc_mode_sense_10(CdIo_t *p_cdio, void *p_buf, unsigned int i_size, 
-		  unsigned int page)
+                  unsigned int page)
 {
     MMC_CMD_SETUP_READ16(CDIO_MMC_GPCMD_MODE_SENSE_10);
     cdb.field[2] = CDIO_MMC_ALL_PAGES & page;
@@ -164,8 +185,8 @@ mmc_mode_sense_6(CdIo_t *p_cdio, void *p_buf, unsigned int i_size, int page)
 */
 driver_return_code_t 
 mmc_prevent_allow_medium_removal(const CdIo_t *p_cdio, 
-				 bool b_persistent, bool b_prevent,
-				 unsigned int i_timeout_ms)
+                                 bool b_persistent, bool b_prevent,
+                                 unsigned int i_timeout_ms)
 {
     uint8_t buf[8] = { 0, };
     void   *p_buf  = &buf; 
@@ -287,18 +308,21 @@ mmc_prevent_allow_medium_removal(const CdIo_t *p_cdio,
 */
 driver_return_code_t
 mmc_read_cd(const CdIo_t *p_cdio, void *p_buf1, lsn_t i_lsn, 
-	    int read_sector_type, bool b_digital_audio_play,
-	    bool b_sync, uint8_t header_codes, bool b_user_data, 
-	    bool b_edc_ecc, uint8_t c2_error_information, 
-	    uint8_t subchannel_selection, uint16_t i_blocksize, 
-	    uint32_t i_blocks)
+            int read_sector_type, bool b_digital_audio_play,
+            bool b_sync, uint8_t header_codes, bool b_user_data, 
+            bool b_edc_ecc, uint8_t c2_error_information, 
+            uint8_t subchannel_selection, uint16_t i_blocksize, 
+            uint32_t i_blocks)
 {
     void *p_buf = p_buf1;
     uint8_t cdb9 = 0;
     const unsigned int i_timeout = mmc_timeout_ms * (MAX_CD_READ_BLOCKS/2);
-    
+
     MMC_CMD_SETUP(CDIO_MMC_GPCMD_READ_CD);
-    
+
+    /* Catch what may be a common bug. */
+    if (NULL == p_buf) return DRIVER_OP_BAD_POINTER;
+
     CDIO_MMC_SET_READ_TYPE(cdb.field, read_sector_type);
     if (b_digital_audio_play) cdb.field[1] |= 0x2;
     
@@ -315,21 +339,21 @@ mmc_read_cd(const CdIo_t *p_cdio, void *p_buf1, lsn_t i_lsn,
       int i_status = DRIVER_OP_SUCCESS;
       
       while (i_blocks > 0) {
-	  const unsigned i_blocks2 = (i_blocks > MAX_CD_READ_BLOCKS) 
-	      ? MAX_CD_READ_BLOCKS : i_blocks;
-	  
-	  const unsigned int i_size = i_blocksize * i_blocks2;
-	  
-	  p_buf = ((char *)p_buf1 ) + (j * i_blocksize);
-	  CDIO_MMC_SET_READ_LBA     (cdb.field, (i_lsn+j));
-	  CDIO_MMC_SET_READ_LENGTH24(cdb.field, i_blocks2);
-	  
-	  i_status = MMC_RUN_CMD(SCSI_MMC_DATA_READ, i_timeout);
-	  
-	  if (i_status) return i_status;
-	  
-	  i_blocks -= i_blocks2;
-	  j += i_blocks2;
+          const unsigned i_blocks2 = (i_blocks > MAX_CD_READ_BLOCKS) 
+              ? MAX_CD_READ_BLOCKS : i_blocks;
+          
+          const unsigned int i_size = i_blocksize * i_blocks2;
+          
+          p_buf = ((char *)p_buf1 ) + (j * i_blocksize);
+          CDIO_MMC_SET_READ_LBA     (cdb.field, (i_lsn+j));
+          CDIO_MMC_SET_READ_LENGTH24(cdb.field, i_blocks2);
+          
+          i_status = MMC_RUN_CMD(SCSI_MMC_DATA_READ, i_timeout);
+          
+          if (i_status) return i_status;
+          
+          i_blocks -= i_blocks2;
+          j += i_blocks2;
       }
       return i_status;
   }
@@ -351,9 +375,9 @@ mmc_read_cd(const CdIo_t *p_cdio, void *p_buf1, lsn_t i_lsn,
  */
 driver_return_code_t 
 mmc_read_disc_information(const CdIo_t *p_cdio, /*out*/ void *p_buf,
-			  unsigned int i_size, 
-			  cdio_mmc_read_disc_info_datatype_t data_type,
-			  unsigned int i_timeout_ms)
+                          unsigned int i_size, 
+                          cdio_mmc_read_disc_info_datatype_t data_type,
+                          unsigned int i_timeout_ms)
 {
     MMC_CMD_SETUP(CDIO_MMC_GPCMD_READ_DISC_INFO);
     CDIO_MMC_SET_READ_LENGTH8(cdb.field, i_size);
@@ -394,10 +418,10 @@ mmc_set_speed(const CdIo_t *p_cdio, int i_Kbs_speed, unsigned int i_timeout_ms)
     uint8_t buf[14] = { 0, };
     void * p_buf = &buf;
     const unsigned int i_size = sizeof(buf);
-    if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
-    
     MMC_CMD_SETUP(CDIO_MMC_GPCMD_SET_SPEED);
-    
+
+    if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
+
     /* If the requested speed is less than 1x 176 kb/s this command
        will return an error - it's part of the ATAPI specs. Therefore, 
        test and stop early. */
@@ -427,7 +451,7 @@ mmc_set_speed(const CdIo_t *p_cdio, int i_Kbs_speed, unsigned int i_timeout_ms)
 */
 driver_return_code_t
 mmc_start_stop_unit(const CdIo_t *p_cdio, bool b_eject, bool b_immediate,
-		    uint8_t power_condition, unsigned int i_timeout_ms)
+                    uint8_t power_condition, unsigned int i_timeout_ms)
 {
   uint8_t buf[1];
   void * p_buf = &buf;
@@ -461,7 +485,105 @@ mmc_test_unit_ready(const CdIo_t *p_cdio, unsigned int i_timeout_ms)
 {
     const unsigned int i_size = 0;
     void  * p_buf = NULL;
-    if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
     MMC_CMD_SETUP_READ16(CDIO_MMC_GPCMD_TEST_UNIT_READY);
+
+    if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
     return MMC_RUN_CMD(SCSI_MMC_DATA_NONE, i_timeout_ms);
 }
+
+/**
+   Issue a READ SUB-CHANNEL command to read current position, ISRC or
+   MCN from subchannel Q.
+   Note: READ SUB-CHANNEL is deprecated as of MMC-5
+         but the alternative requires manual parsing of the subchannel.
+
+   @param p_cdio  the CD object to be acted upon.
+   @param i_track track number (only for ISRC)
+   @param sub_chan_param 2 for MCN, 3 for ISRC
+   @param i_length pointer to number of bytes to request. 
+                   Will be overwritten by the number of bytes available.
+   @param p_buf  pointer to the location for the returned data
+
+   @return DRIVER_OP_SUCCESS on success
+ */
+driver_return_code_t
+mmc_read_subchannel ( const CdIo_t *p_cdio,
+                             track_t i_track,
+                             unsigned char sub_chan_param,
+                             unsigned int *i_length,
+                             char *p_buf,
+                             unsigned int i_timeout_ms
+                            )
+{
+  unsigned int i_size = *i_length;
+  mmc_cdb_t cdb = {{0, }};
+  driver_return_code_t i_status;
+
+  if (i_size < 4)
+    return DRIVER_OP_BAD_PARAMETER;
+
+  CDIO_MMC_SET_COMMAND(cdb.field, CDIO_MMC_GPCMD_READ_SUBCHANNEL);
+  CDIO_MMC_SET_READ_LENGTH8(cdb.field, i_size);
+
+  if(CDIO_SUBCHANNEL_CURRENT_POSITION == sub_chan_param)
+    cdb.field[1] = CDIO_CDROM_MSF;
+  else
+    cdb.field[1] = 0x0;
+  cdb.field[2] = 0x40;
+  cdb.field[3] = sub_chan_param;
+  if(CDIO_SUBCHANNEL_TRACK_ISRC == sub_chan_param)
+   cdb.field[6] = i_track;
+  else
+   cdb.field[6] = 0;
+
+  memset(p_buf, 0, i_size);
+
+  if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
+
+  i_status = MMC_RUN_CMD(SCSI_MMC_DATA_READ, i_timeout_ms);
+
+  if(i_status == DRIVER_OP_SUCCESS) {
+    *i_length = CDIO_MMC_GET_LEN16((p_buf+2)) + 4;
+  }
+  return i_status;
+}
+
+/**
+   Issue a READ TOC/PMA/ATIP command to read the CD-TEXT from R-W sub-channel.
+
+   @param p_cdio  the CD object to be acted upon.
+   @param i_length pointer to number of bytes to request. 
+                   Will be overwritten by the number of bytes available.
+   @param p_buf  pointer to the location for the returned data
+
+   @return DRIVER_OP_SUCCESS on success
+ */
+driver_return_code_t
+mmc_read_toc_cdtext ( const CdIo_t *p_cdio, unsigned int *i_length,
+                      unsigned char *p_buf, unsigned int i_timeout_ms )
+{
+  unsigned int i_size = *i_length;
+  mmc_cdb_t  cdb = {{0, }};
+  driver_return_code_t i_status;
+
+  if (i_size < 4)
+    return DRIVER_OP_BAD_PARAMETER;
+
+  CDIO_MMC_SET_COMMAND(cdb.field, CDIO_MMC_GPCMD_READ_TOC);
+  CDIO_MMC_SET_READ_LENGTH16(cdb.field, i_size);
+
+  memset(p_buf, 0, i_size);
+
+  if (0 == i_timeout_ms) i_timeout_ms = mmc_timeout_ms;
+
+  cdb.field[1] = CDIO_CDROM_MSF;
+  cdb.field[2] = CDIO_MMC_READTOC_FMT_CDTEXT;
+
+  i_status = MMC_RUN_CMD(SCSI_MMC_DATA_READ, i_timeout_ms);
+
+  if(i_status == DRIVER_OP_SUCCESS) {
+    *i_length = CDIO_MMC_GET_LEN16(p_buf) + 4;
+  }
+
+  return i_status;
+}
