@@ -1,6 +1,6 @@
 /* -*- C -*-
   Copyright (C) 2012 Rocky Bernstein <rocky@gnu.org>
-  
+
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* 
+/*
    Unit test for cdda-like things lib/driver/disc.c
 */
 #ifdef HAVE_CONFIG_H
@@ -40,8 +40,9 @@
 int
 main(int argc, const char *argv[])
 {
-  char *mcn = cdio_get_mcn(NULL);
-  
+  char *mcn;
+  cdio_get_mcn(NULL);
+
   cdio_log_set_handler(log_handler);
   cdio_loglevel_default = (argc > 1) ? CDIO_LOG_DEBUG : CDIO_LOG_INFO;
 
@@ -51,6 +52,6 @@ main(int argc, const char *argv[])
       printf("Should have gotten NULL for MCN for cdio object\n");
       return 1;
   }
-  
+
   return 0;
 }
